import numpy as np  #soporte para arreglos multidimensionales
import cv2

def order_points(pts): #pts es una lista de 4 puntos con coordenadas xy
    rect = np.zeros((4, 2), dtype="float32") # reserva espacio para guardar las coordenadas
    #suma de xy
    s = pts.sum(axis=1)
    rect[0] = pts[np.argmin(s)]  # suma de xy  esquina tl
    rect[2] = pts[np.argmax(s)]  # suma de xy esquina br
    #resta de xy
    diff = np.diff(pts, axis=1)
    rect[1] = pts[np.argmin(diff)]  #busca las esquinas opuestas haciendo una resta  tr
    rect[3] = pts[np.argmax(diff)]  # resta para esquina bl

    return rect


def four_point_transform(image, pts):
	# obtain a consistent order of the points and unpack them
	# individually
	rect = order_points(pts)
	(tl, tr, br, bl) = rect
	# compute the width of the new image, which will be the
	# maximum distance between bottom-right and bottom-left
	# x-coordiates or the top-right and top-left x-coordinates
	widthA = np.sqrt(((br[0] - bl[0]) ** 2) + ((br[1] - bl[1]) ** 2))
	widthB = np.sqrt(((tr[0] - tl[0]) ** 2) + ((tr[1] - tl[1]) ** 2))
	maxWidth = max(int(widthA), int(widthB))
	# compute the height of the new image, which will be the
	# maximum distance between the top-right and bottom-right
	# y-coordinates or the top-left and bottom-left y-coordinates
	heightA = np.sqrt(((tr[0] - br[0]) ** 2) + ((tr[1] - br[1]) ** 2))
	heightB = np.sqrt(((tl[0] - bl[0]) ** 2) + ((tl[1] - bl[1]) ** 2))
	maxHeight = max(int(heightA), int(heightB))
	# now that we have the dimensions of the new image, construct
	# the set of destination points to obtain a "birds eye view",
	# (i.e. top-down view) of the image, again specifying points
	# in the top-left, top-right, bottom-right, and bottom-left
	# order
	dst = np.array([
		[0, 0],
		[maxWidth - 1, 0],
		[maxWidth - 1, maxHeight - 1],
		[0, maxHeight - 1]], dtype = "float32")
	# compute the perspective transform matrix and then apply it
	M = cv2.getPerspectiveTransform(rect, dst)
	warped = cv2.warpPerspective(image, M, (maxWidth, maxHeight))
	# return the warped image
	return warped



import numpy as np
import argparse
import imutils
import cv2

def sort_contours(cnts, method="left-to-right"):
	# initialize the reverse flag and sort index
	reverse = False
	i = 0
	# handle if we need to sort in reverse
	if method == "right-to-left" or method == "bottom-to-top":
		reverse = True
	# handle if we are sorting against the y-coordinate rather than
	# the x-coordinate of the bounding box
	if method == "top-to-bottom" or method == "bottom-to-top":
		i = 1
	# construct the list of bounding boxes and sort them from top to
	# bottom
	boundingBoxes = [cv2.boundingRect(c) for c in cnts]
	(cnts, boundingBoxes) = zip(*sorted(zip(cnts, boundingBoxes),
		key=lambda b:b[1][i], reverse=reverse))
	# return the list of sorted contours and bounding boxes
	return (cnts, boundingBoxes)




from transformar import four_point_transform
#from imutils.perspective import four_point_transform
from contornos import sort_contours
import numpy as np
import argparse
import imutils
import cv2
import pytesseract
import pyttsx

pytesseract.pytesseract.tesseract_cmd = '/usr/bin/tesseract'  # permite invocar tesseract desde python

imagen = cv2.imread('foto4.jpg')  # se carga la imagen

escalaGrises = cv2.cvtColor(imagen, cv2.COLOR_BGR2GRAY)  # convierte a escala de grises
suavizado = cv2.GaussianBlur(escalaGrises,(5,5),0)
umbral = cv2.adaptiveThreshold(escalaGrises, 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 91, 11)

bordes = cv2.Canny(suavizado, 75, 200)
#===========================CONTORNOS====================================

cnts = cv2.findContours(bordes.copy(), cv2.RETR_EXTERNAL,
	cv2.CHAIN_APPROX_SIMPLE)
cnts = imutils.grab_contours(cnts)
docCnt = None

#==============================
if len(cnts) > 0:
	# sort the contours according to their size in
	# descending order
	cnts = sorted(cnts, key=cv2.contourArea, reverse=True)
	# loop over the sorted contours
	for c in cnts:
		# approximate the contour
		peri = cv2.arcLength(c, True)
		approx = cv2.approxPolyDP(c, 0.02 * peri, True)
		# if our approximated contour has four points,
		# then we can assume we have found the paper
		if len(approx) == 4:
			docCnt = approx
			break
hoja = four_point_transform(escalaGrises, docCnt.reshape(4, 2))
warped = four_point_transform(escalaGrises, docCnt.reshape(4, 2))

#===========================Umbral adaptivo====================================


thresh = cv2.threshold(warped, 0, 255,
	cv2.THRESH_BINARY_INV | cv2.THRESH_OTSU)[1]



#=====================================================================C===

#grados de precision  thres ->
texto = pytesseract.image_to_string(hoja)
print(texto)

engine = pyttsx.init('espeak')
engine.setProperty('rate', 175)
#voices = engine.getProperty('voices')
#engine.setProperty('voice', voices[1].id)
engine.say(texto)
engine.runAndWait()



"""
#cv2.imshow("1 umbral", umbral)
cv2.imshow("2 warped+thres", thresh)
cv2.imshow("3-warped", warped)
cv2.imshow("4-hoja", hoja)
"""

cv2.waitKey(0)
